\name{add.constraint}
\title{
 Add a constraint to an triangulaion object
}
\alias{add.constraint}
\usage{
add.constraint(tri.obj,cstx,csty,reverse=FALSE)
}
\arguments{
  \item{tri.obj}{object of class \code{"tri"}}
  \item{cstx}{vector containing x coordinates of the constraint curve.}
  \item{csty}{vector containing y coordinates of the constraint curve.}
  \item{reverse}{if \code{TRUE} the orientation of the constraint curve
    is reversed.}
}
\value{
An new object of class \code{"tri"}.}
\description{
  This subroutine provides for creation of a constrained
Delaunay triangulation which, in some sense, covers an
arbitrary connected region R rather than the convex hull
of the nodes.  This is achieved simply by forcing the
presence of certain adjacencies (triangulation arcs) corresponding
to constraint curves.  The union of triangles
coincides with the convex hull of the nodes, but triangles
in R can be distinguished from those outside of R.  The
only modification required to generalize the definition of
the Delaunay triangulation is replacement of property 5
(refer to \code{\link{tri.mesh}} by the following:

5')  If a node is contained in the interior of the circumcircle
of a triangle, then every interior point
of the triangle is separated from the node by a
constraint arc.

In order to be explicit, we make the following definitions.
A constraint region is the open interior of a
simple closed positively oriented polygonal curve defined
by an ordered sequence of three or more distinct nodes
(constraint nodes) P(1),P(2),...,P(K), such that P(I) is
adjacent to P(I+1) for I = 1,...,K with P(K+1) = P(1).
Thus, the constraint region is on the left (and may have
nonfinite area) as the sequence of constraint nodes is
traversed in the specified order.  The constraint regions
must not contain nodes and must not overlap.  The region
R is the convex hull of the nodes with constraint regions
excluded.

  Note that the terms boundary node and boundary arc are
reserved for nodes and arcs on the boundary of the convex
hull of the nodes.

  The algorithm is as follows:  given a triangulation
which includes one or more sets of constraint nodes, the
corresponding adjacencies (constraint arcs) are forced to
be present (Fortran subroutine EDGE).  Any additional new arcs
required are chosen to be locally optimal (satisfy the
modified circumcircle property).
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}

\seealso{
  \code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}}, \code{\link{summary.tri}}, \code{\link{triangles}}, \code{\link{convex.hull}}.
}
\examples{
# we will use the simple test data from TRIPACK:
data(tritest)
tritest.tr<-tri.mesh(tritest)
opar<-par(mfrow=c(2,2))
plot(tritest.tr)
# include all points in a big triangle:
tritest.tr<-add.constraint(tritest.tr,c(-0.1,2,-0.1),
                                      c(-3,0.5,3),reverse=TRUE)
# insert a small cube:
tritest.tr <- add.constraint(tritest.tr, c(0.4, 0.4,0.6, 0.6), 
                                         c(0.6, 0.4,0.4, 0.6), 
                                         reverse = FALSE)
par(opar)
}
\keyword{spatial}

\eof
\name{cells}
\alias{cells}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ extract info about voronoi cells }
\description{
  This function returns some info about the cells of a voronoi mosaic,
  including the coordinates of the vertices and the cell area.
}
\usage{
cells(voronoi.obj)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{voronoi.obj}{ object of class \code{voronoi}  }
}
\details{
  The function calculates the neighbourhood relations between the
  underlying triangulation and translates it into the neighbourhood
  relations between the voronoi cells.
}
\value{
  retruns a list of lists, one entry for each voronoi cell which contains
  \item{cell }{cell index}
  \item{center }{cell 'center'}
  \item{neighbours }{neighbour cell indices}
  \item{nodes}{2 times \code{nnb} matrix with vertice coordinates}
  \item{area}{cell area}
}

\author{ A. Gebhardt }
\note{ outer cells have \code{area=NA}, currently also \code{nodes=NA}
  which is not really useful -- to be done later }


\seealso{  \code{\link{voronoi.mosaic}},  \code{\link{voronoi.area}} }
\examples{
data(tritest)
tritest.vm <- voronoi.mosaic(tritest$x,tritest$y)
tritest.cells <- cells(tritest.vm)
# higlight cell 12:
plot(tritest.vm)
polygon(t(tritest.cells[[12]]$nodes),col="green")
# put cell area into cell center:
text(tritest.cells[[12]]$center[1],
     tritest.cells[[12]]$center[2],
     tritest.cells[[12]]$area)
}
\keyword{ spatial }% at least one, from doc/KEYWORDS

\eof
\name{convex.hull}
\title{Return the convex hull of a triangulation object}
\author{A. Gebhardt}
\usage{convex.hull(tri.obj, plot.it=FALSE, add=FALSE,...)
}
\alias{convex.hull}
\arguments{
  \item{tri.obj}{object of class \code{"tri"}}
  \item{plot.it}{logical, if \code{TRUE} the convex hull of
    \code{tri.obj} will be plotted.}
  \item{add}{logical. if \code{TRUE} (and \code{plot.it=TRUE}), add to
    a current plot.}
  \item{...}{additional plot arguments}
}
\description{
Given a triangulation \code{tri.obj} of \eqn{n} points in the plane, this
subroutine returns two vectors containing the coordinates
of the nodes on the boundary of the convex hull.

}
\value{
  \item{x}{x coordinates of boundary nodes.}
  \item{y}{y coordinates of boundary nodes.}
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}}, \code{\link{summary.tri}}, \code{\link{triangles}}, \code{\link{add.constraint}}.
}
\examples{
# rather simple example from TRIPACK:
data(tritest)
tr<-tri.mesh(tritest$x,tritest$y)
convex.hull(tr,plot.it=TRUE)
# random points:
rand.tr<-tri.mesh(runif(10),runif(10))
plot(rand.tr)
rand.ch<-convex.hull(rand.tr, plot.it=TRUE, add=TRUE, col="red")
# use a part of the quakes data set:
data(quakes)
quakes.part<-quakes[(quakes[,1]<=-17 & quakes[,1]>=-19.0 &
                     quakes[,2]<=182.0 & quakes[,2]>=180.0),]
quakes.tri<-tri.mesh(quakes.part$lon, quakes.part$lat, duplicate="remove")
plot(quakes.tri)
convex.hull(quakes.tri, plot.it=TRUE, add=TRUE, col="red")
}
\keyword{spatial}

\eof
\name{identify.tri}
\title{Identify points in a triangulation plot}
\author{A. Gebhardt}
\usage{identify.tri(x,...)
}
\alias{identify.tri}
\arguments{
  \item{x}{object of class \code{"tri"}}
  \item{...}{additional paramters for \code{identify}}
}
\description{Identify points in a plot of \code{"x"} with its
  coordinates. The plot of \code{"x"} must be generated with \code{plot.tri}.
}
\value{an integer vector containing the indexes of the identified points.
}
\seealso{
    \code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}}, \code{\link{summary.tri}}
}
\examples{
data(tritest)
tritest.tr<-tri.mesh(tritest$x,tritest$y)
plot(tritest.tr)
identify.tri(tritest.tr)
}
\keyword{spatial}

\eof
\name{in.convex.hull}
\title{Determines if points are in the convex hull of a triangulation object}
\author{A. Gebhardt}
\usage{in.convex.hull(tri.obj, x, y)
}
\alias{in.convex.hull}
\arguments{
  \item{tri.obj}{object of class \code{"tri"}}
  \item{x}{vector of x-coordinates of points to locate}
  \item{y}{vector of y-coordinates of points to locate}
}
\description{
Given a triangulation \code{tri.obj} of \eqn{n} points in the plane, this
subroutine returns a logical vector indicating if the points
\eqn{(x_i,y_i)}
are contained within the convex hull of \code{tri.obj}.

}
\value{
Logical vector.
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}},
  \code{\link{summary.tri}}, \code{\link{triangles}},
  \code{\link{add.constraint}}, \code{\link{convex.hull}}.
}
\examples{
# example from TRIPACK:
data(tritest)
tr<-tri.mesh(tritest$x,tritest$y)
in.convex.hull(tr,0.5,0.5)
in.convex.hull(tr,c(0.5,-1,1),c(0.5,1,1))
# use a part of the quakes data set:
data(quakes)
quakes.part<-quakes[(quakes[,1]<=-10.78 & quakes[,1]>=-19.4 &
                     quakes[,2]<=182.29 & quakes[,2]>=165.77),]
q.tri<-tri.mesh(quakes.part$lon, quakes.part$lat, duplicate="remove")
in.convex.hull(q.tri,quakes$lon[990:1000],quakes$lat[990:1000])
}
\keyword{spatial}

\eof
\name{neighbours}
\title{List of neighbours from a triangulation object}
\author{A. Gebhardt}
\usage{neighbours(tri.obj)
}
\alias{neighbours}
\arguments{
  \item{tri.obj}{object of class \code{"tri"}}
}
\description{Extract a list of neighbours from a triangulation object
}
\value{nested list of neighbours per point
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{\code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}}, \code{\link{summary.tri}}, \code{\link{triangles}}
}
\examples{
data(tritest)
tritest.tr<-tri.mesh(tritest$x,tritest$y)
tritest.nb<-neighbours(tritest.tr)
}
\keyword{spatial}

\eof
\name{on.convex.hull}
\title{Determines if points are on the convex hull of a triangulation object}
\author{A. Gebhardt}
\usage{on.convex.hull(tri.obj, x, y)
}
\alias{on.convex.hull}
\arguments{
  \item{tri.obj}{object of class \code{"tri"}}
  \item{x}{vector of x-coordinates of points to locate}
  \item{y}{vector of y-coordinates of points to locate}
}
\description{
Given a triangulation \code{tri.obj} of \eqn{n} points in the plane, this
subroutine returns a logical vector indicating if the points
\eqn{(x_i,y_i)}
lay on the convex hull of \code{tri.obj}.

}
\value{
Logical vector.
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}},
  \code{\link{summary.tri}}, \code{\link{triangles}},
  \code{\link{add.constraint}}, \code{\link{convex.hull}}, \code{\link{in.convex.hull}}.
}
\examples{
# example from TRIPACK:
data(tritest)
tr<-tri.mesh(tritest$x,tritest$y)
on.convex.hull(tr,0.5,0.5)
on.convex.hull(tr,c(0.5,-1,1),c(0.5,1,1))
# use a part of the quakes data set:
data(quakes)
quakes.part<-quakes[(quakes[,1]<=-10.78 & quakes[,1]>=-19.4 &
                     quakes[,2]<=182.29 & quakes[,2]>=165.77),]
q.tri<-tri.mesh(quakes.part$lon, quakes.part$lat, duplicate="remove")
on.convex.hull(q.tri,quakes.part$lon[1:20],quakes.part$lat[1:20])
}
\keyword{spatial}

\eof
\name{outer.convhull}
\title{Version of outer which operates only in a convex hull}
\author{A. Gebhardt}
\usage{outer.convhull(cx,cy,px,py,FUN,duplicate="remove",...)
}
\alias{outer.convhull}
\arguments{
  \item{cx}{x cordinates of grid}
  \item{cy}{y cordinates of grid}
  \item{px}{vector of x coordinates of points}
  \item{py}{vector of y coordinates of points}
  \item{FUN}{function to be evaluated over the grid}
  \item{duplicate}{indicates what to do with duplicate
    \eqn{(px_i,py_i)} points, default \code{"remove"}.}
  \item{...}{additional arguments for \code{FUN}}
}
\description{This version of \code{outer} evaluates \code{FUN}
  only on that part of the grid \eqn{cx x cy} that is enclosed within
  the convex hull of the points (\code{px},\code{py}).
  
  This can be useful for spatial estimation if no extrapolation is wanted. 
}
\value{Matrix with values of \code{FUN} (\code{NA}s if outside the
  convex hull).
}
\seealso{\code{\link{in.convex.hull}}
}
\examples{
x<-runif(20)
y<-runif(20)
z<-runif(20)
z.lm<-lm(z~x+y)
f.pred<-function(x,y)
  {predict(z.lm,data.frame(x=as.vector(x),y=as.vector(y)))}
xg<-seq(0,1,0.05)
yg<-seq(0,1,0.05)
image(xg,yg,outer.convhull(xg,yg,x,y,f.pred))
points(x,y)
}
\keyword{spatial}


\eof
\name{plot.tri}
\title{Plot a triangulation object}
\author{A. Gebhardt}
\usage{plot.tri(x, add=FALSE,xlim=range(x$x),ylim=range(x$y), do.points=TRUE, ...)
}
\alias{plot.tri}
\arguments{
  \item{x}{object of class \code{"tri"}}
  \item{add}{logical, if \code{TRUE}, add to a current plot.}
  \item{do.points}{logical, indicates if points should be plotted.}
  \item{xlim,ylim}{x/y ranges for plot}
  \item{...}{additional plot parameters}
}
\description{plots the triangulation \code{"x"}
}
\value{None
}
\references{
   R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8. 
}
\seealso{
  \code{\link{tri}}, \code{\link{print.tri}}, 
  \code{\link{summary.tri}}
}
\examples{
# random points
plot(tri.mesh(rpois(100,lambda=20),rpois(100,lambda=20),duplicate="remove"))
# use a part of the quakes data set:
data(quakes)
quakes.part<-quakes[(quakes[,1]<=-10.78 & quakes[,1]>=-19.4 &
                     quakes[,2]<=182.29 & quakes[,2]>=165.77),]
quakes.tri<-tri.mesh(quakes.part$lon, quakes.part$lat, duplicate="remove")
plot(quakes.tri)
# use the whole quakes data set 
# (will not work with standard memory settings, hence commented out)
#plot(tri.mesh(quakes$lon, quakes$lat, duplicate="remove"), do.points=F) 
}
\keyword{spatial}

\eof
\name{plot.voronoi}
\title{Plot a xect}
\author{A. Gebhardt}
\usage{plot.voronoi(x, add=FALSE, xlim=c(min(x$tri$x)-0.1*diff(range(x$tri$x)),   ylim=c(min(x$tri$y)-0.1*diff(range(x$tri$y)),max(x$tri$y)+0.1*diff(range(x$tri$y))),all=FALSE,do.points=TRUE,main="Voronoi mosaic",sub=deparse(substitute(x), ...)
)
}
\alias{plot.voronoi}
\arguments{
  \item{x}{object of class \code{"voronoi"}}
  \item{add}{logical, if \code{TRUE}, add to a current plot.}
  \item{do.points}{logical, indicates if points should be plotted.}
  \item{xlim,ylim}{x/y plot ranges, by default modified to hide dummy 
        points outside of the plot}
  \item{all}{show all (including dummy points in the plot}
  \item{main,sub}{plot titles}
  \item{...}{additional plot parameters}
}
\description{Plots the mosaic \code{"x"}.

  Dashed lines are used for outer tiles of the mosaic.
}
\value{None
}
\references{
   R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8. 
}
\seealso{
  \code{\link{voronoi}}, \code{\link{print.voronoi}}, 
  \code{\link{summary.voronoi}}
}
\examples{
# generate a random mosaic
plot(voronoi.mosaic(runif(100),runif(100),duplicate="remove"))
# use a part of the quakes data set:
data(quakes)
quakes.part<-quakes[(quakes[,1]<=-17 & quakes[,1]>=-19.0 &
                     quakes[,2]<=182.0 & quakes[,2]>=180.0),]
quakes.vm<-voronoi.mosaic(quakes.part$lon, quakes.part$lat, duplicate="remove")
plot(quakes.vm)
# use the whole quakes data set 
# (will not work with standard memory settings, hence commented out here)
#plot(voronoi.mosaic(quakes$lon, quakes$lat, duplicate="remove"))
}
\keyword{spatial}

\eof
\name{plot.voronoi.polygons}
\alias{plot.voronoi.polygons}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{plots an voronoi.polygons object}
\description{
  plots an \code{voronoi.polygons} object
}
\usage{
plot.voronoi.polygons(x, which, color=TRUE, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{ object of class \code{voronoi.polygons}  }
  \item{which}{ index vector selecting which polygons to plot }
  \item{color}{ logical, determines if plot should be colored, default: \code{TRUE} }
  \item{\dots}{ additional plot arguments }
}
\author{ A. Gebhardt}

\seealso{ \code{\link{voronoi.polygons}}}

\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	     or do  help(data=index)  for the standard data sets.
data(tritest)
tritest.vm <- voronoi.mosaic(tritest$x,tritest$y)
tritest.vp <- voronoi.polygons(tritest.vm)
plot(tritest.vp)
plot(tritest.vp,which=c(1,3,5))
}
\keyword{ spatial }% at least one, from doc/KEYWORDS

\eof
\name{print.summary.tri}
\title{Print a summary of a triangulation object}
\author{A. Gebhardt}
\usage{print.summary.tri(x, ...)}
\alias{print.summary.tri}
\arguments{
  \item{x}{object of class \code{"summary.tri"},
    generated by \code{\link{summary.tri}}.}
  \item{...}{additional paramters for \code{print}}
}
\description{
  Prints some information about \code{tri.obj}
}
\value{None
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{tri}},\code{\link{tri.mesh}},
  \code{\link{print.tri}}, \code{\link{plot.tri}},
  \code{\link{summary.tri}}. 
}
\keyword{spatial}

\eof
\name{print.summary.voronoi}
\title{Print a summary of a voronoi object}
\author{A. Gebhardt}
\usage{print.summary.voronoi(x, ...)}
\alias{print.summary.voronoi}
\arguments{
  \item{x}{object of class \code{"summary.voronoi"},
    generated by \code{\link{summary.voronoi}}.}
  \item{...}{additional paramters for \code{print}}
}
\description{
  Prints some information about \code{x}
}
\value{None
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{voronoi}},\code{\link{voronoi.mosaic}},
  \code{\link{print.voronoi}}, \code{\link{plot.voronoi}},
  \code{\link{summary.voronoi}}. 
}
\keyword{spatial}

\eof
\name{print.tri}
\title{Print a triangulation object}
\author{A. Gebhardt}
\usage{print.tri(x,...)
}
\alias{print.tri}
\arguments{
  \item{x}{object of class \code{"tri"}}
  \item{...}{additional paramters for \code{print}}
}
\description{prints a adjacency list of \code{"x"}
}
\value{None
}
\references{
    R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{tri}},
  \code{\link{plot.tri}},
  \code{\link{summary.tri}}
}
\keyword{spatial}

\eof
\name{print.voronoi}
\title{Print a xect}
\author{A. Gebhardt}
\usage{print.voronoi(x,...)
}
\alias{print.voronoi}
\arguments{
  \item{x}{object of class \code{"voronoi"}}
  \item{...}{additional paramters for \code{print}}
}
\description{prints a summary of \code{"x"}
}
\value{None
}
\references{
    R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{voronoi}},
  \code{\link{plot.voronoi}},
  \code{\link{summary.voronoi}}
}
\keyword{spatial}

\eof
\name{summary.tri}
\title{Return a summary of a triangulation object}
\author{A. Gebhardt}
\usage{summary.tri(object,...)
}
\alias{summary.tri}
\arguments{
  \item{object}{object of class \code{"tri"}}
  \item{...}{additional paramters for \code{summary}}
}
\description{
  Returns some information (number of nodes, triangles, arcs, boundary
  nodes and constraints) about \code{object}.
}
\value{An objekt of class \code{"summary.tri"}, to be printed by
  \code{\link{print.summary.tri}}.

  It contains the number of nodes (\code{n}), of arcs (\code{na}), of
  boundary nodes (\code{nb}), of triangles (\code{nt}) and constraints
  (\code{nc}).   
}
\references{
    R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}},
  \code{\link{print.summary.tri}}.
}
\keyword{spatial}

\eof
\name{summary.voronoi}
\title{Return a summary of a objectect}
\author{A. Gebhardt}
\usage{summary.voronoi(object,...)
}
\alias{summary.voronoi}
\arguments{
  \item{object}{object of class \code{"voronoi"}}
  \item{...}{additional paramters for \code{summary}}
}
\description{
  Returns some information about \code{object}
}
\value{Object of class \code{"summary.voronoi"}.

  It contains the number of nodes (\code{nn}) and dummy nodes (\code{nd}).
}
\references{
    R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{voronoi}},\code{\link{voronoi.mosaic}},
  \code{\link{print.voronoi}}, \code{\link{plot.voronoi}},
  \code{\link{print.summary.voronoi}}. 
}
\keyword{spatial}

\eof
\name{tri}
\title{A triangulation object}
\author{A. Gebhardt}
\alias{tri}
\description{
  R object that represents the triangulation of a set of 2D points, 
  generated by \code{\link{tri.mesh}} or \code{\link{add.constraint}}.
}
\arguments{
  \item{n}{Number of nodes}
  \item{x}{x coordinates of the triangulation nodes}
  \item{y}{y coordinates of the triangulation nodes}
  \item{tlist}{Set of nodal indexes which, along with \code{tlptr},
    \code{tlend}, and \code{tlnew}, define the triangulation as a
    set of \eqn{n} adjacency lists -- counterclockwise-ordered sequences of
    neighboring nodes such that the first and last neighbors of a
    boundary node are boundary nodes (the first neighbor of an interior
    node is arbitrary). In order to distinguish between
    interior and boundary nodes, the last neighbor of each boundary node
    is represented by the negative of its index.}
  \item{tlptr}{Set of pointers in one-to-one
    correspondence with the elements of \code{tlist}.
    \code{tlist[tlptr[i]]} indexes the node which follows
    \code{tlist[i]} in cyclical counterclockwise order
    (the first neighbor follows the last neighbor).}
  \item{tlend}{Set of pointers to adjacency lists. \code{tlend[k]}
    points to the last neighbor of node \eqn{k} for
    \eqn{k = 1,...,n}.  Thus, \code{tlist[\tlend[k]]}<0 if and
    only if \eqn{k} is a boundary node.}
  \item{tlnew}{ Pointer to the first empty location in \code{tlist}
    and \code{tlptr} (list length plus one).}
  \item{nc}{number of constraints}
  \item{lc}{starting indices of constraints in \code{x} and \code{y}}
  \item{call}{call, which generated this object}
}
\note{
The elements \code{tlist}, \code{tlptr}, \code{tlend} and \code{tlnew}
are mainly intended for internal use in the appropriate Fortran
routines. 
}
\references{
    R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{\code{\link{tri.mesh}}, \code{\link{print.tri}}, \code{\link{plot.tri}}, \code{\link{summary.tri}}
}
\keyword{spatial}

\eof
\name{tri.dellens}
\title{Compute the Delaunay segment lengths}
\author{S. J. Eglen}
\usage{
tri.dellens(voronoi.obj, exceptions = NULL, inverse = FALSE)
}
\alias{tri.dellens}
\arguments{
  \item{voronoi.obj}{object of class \code{"voronoi"}}
  \item{exceptions}{a numerical vector}
  \item{inverse}{Logical}
}
\description{
  Return a vector of Delaunay segment lengths for the voronoi object.
  The Delaunay triangles connected to sites contained in  \code{exceptions}
  vector are ignored (unless \code{inverse} is TRUE, when only those
  Delaunay triangles are accepted).

  The \code{exceptions} vector is provided so that sites at the border
  of a region can be removed, as these tend to bias the distribution of
  Delaunay segment lengths.  \code{exceptions} can be created by
  \code{\link{voronoi.findrejectsites}}.
}
\value{
  A vector of Delaunay segment lengths.
}
\seealso{
  \code{\link{voronoi.findrejectsites}}, \code{\link{voronoi.mosaic}},
}
\examples{
data(tritest)
tritest.vm <- voronoi.mosaic(tritest$x,tritest$y)

tritest.vm.rejects <- voronoi.findrejectsites(tritest.vm, 0,1, 0, 1)
trilens.all <- tri.dellens(tritest.vm)
trilens.acc <- tri.dellens(tritest.vm, tritest.vm.rejects)
trilens.rej <- tri.dellens(tritest.vm, tritest.vm.rejects, inverse=TRUE)

par(mfrow=c(3,1))
dotchart(trilens.all, main="all Delaunay segment lengths")
dotchart(trilens.acc, main="excluding border sites")
dotchart(trilens.rej, main="only border sites")
}
\keyword{spatial}

\eof
\name{tri.find}
\title{Locate a point in a triangulation}
\author{A. Gebhardt}
\usage{
tri.find(tri.obj,x,y)
}
\alias{tri.find}
\arguments{
  \item{tri.obj}{an triangulation object}
    \item{x}{x-coordinate of the point}
    \item{y}{y-coordinate of the point}
}
\description{
  This subroutine locates a point P=(\code{x},\code{y}) relative to a triangulation
  created by \code{tri.mesh}.  If P is
  contained in a triangle, the three vertex indexes are
  returned.  Otherwise, the indexes of the rightmost and
  leftmost visible boundary nodes are returned.
}
\value{
  A list with elements \code{i1},\code{i2},code{i3} containing
  nodal indexes, in counterclockwise order,
  of the vertices of a triangle containing
  P=(\code{x},\code{y}), or, if P is not contained in the convex
  hull of the nodes, \code{i1} indexes the
  rightmost visible boundary node, \code{i2} indexes
  the leftmost visible boundary node,
  and \code{i3} = 0.  Rightmost and leftmost are
  defined from the perspective of P, and a
  pair of points are visible from each
  other if and only if the line segment
  joining them intersects no triangulation
  arc.  If P and all of the nodes lie on a
  common line, then \code{i1}=\code{i2}=\code{i3} = 0 on
  output.
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}},
  \code{\link{summary.tri}}, \code{\link{triangles}}, 
  \code{\link{convex.hull}}
}
\examples{
data(tritest)
tritest.tr<-tri.mesh(tritest$x,tritest$y)
plot(tritest.tr)
pnt<-list(x=0.3,y=0.4)
triangle.with.pnt<-tri.find(tritest.tr,pnt$x,pnt$y)
attach(triangle.with.pnt)
lines(tritest$x[c(i1,i2,i3,i1)],tritest$y[c(i1,i2,i3,i1)],col="red")
points(pnt$x,pnt$y)
}
\keyword{spatial}

\eof
\name{tri.mesh}
\title{
  Create a delaunay triangulation
}
\alias{tri.mesh}
\usage{
tri.mesh(x, y = NULL, duplicate = "error")
}
\arguments{
  \item{x}{vector containing x coordinates of the data. If \code{y} is missing
   \code{x} should contain two elements \code{$x} and \code{$y}.
  }
  \item{y}{vector containing y coordinates of the data.
  }
  \item{duplicate}{flag indicating how to handle duplicate elements.
   Possible values are: \code{"error"} -- default, \code{"strip"} -- remove all duplicate points, \code{"remove"} -- leave one point of duplicate points.
  }
}
\value{
An object of class \code{"tri"}
}
\description{

  This subroutine creates a Delaunay triangulation of a
  set of N arbitrarily distributed points in the plane referred to as
  nodes.
  The Delaunay triangulation is defined
  as a set of triangles with the following five properties:

 1)  The triangle vertices are nodes.

 2)  No triangle contains a node other than its vertices.

 3)  The interiors of the triangles are pairwise disjoint.

 4)  The union of triangles is the convex hull of the set
       of nodes (the smallest convex set which contains
       the nodes).

 5)  The interior of the circumcircle of each triangle
       contains no node.

The first four properties define a triangulation, and the
last property results in a triangulation which is as close
as possible to equiangular in a certain sense and which is
uniquely defined unless four or more nodes lie on a common
circle.  This property makes the triangulation well-suited
for solving closest point problems and for triangle-based
interpolation.

  The triangulation can be generalized to a constrained
Delaunay triangulation by a call to \code{add.constraint}.
This allows for user-specified boundaries defining a nonconvex 
and/or multiply connected region.

  The operation count for constructing the triangulation
is close to O(N) if the nodes are presorted on X or Y components.  
Also, since the algorithm proceeds by adding
nodes incrementally, the triangulation may be updated with
the addition (or deletion) of a node very efficiently.
The adjacency information representing the triangulation
is stored as a linked list requiring approximately 13N
storage locations.
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{
  \code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}},
  \code{\link{summary.tri}}, \code{\link{triangles}}, 
  \code{\link{convex.hull}}, \code{\link{neighbours}}, 
  \code{\link{add.constraint}}.
}
\examples{
data(tritest)
tritest.tr<-tri.mesh(tritest$x,tritest$y)
tritest.tr
}
\keyword{spatial}

\eof
\name{triangles}
\title{Extract a list of triangle from a triangulation object}
\author{A. Gebhardt}
\usage{triangles(tri.obj)
}
\alias{triangles}
\arguments{
  \item{tri.obj}{object of class \code{"tri"}}
}
\description{This function extracts a triangulation data structure
  from an triangulation object created by \code{tri.mesh}.

    The vertices in the returned matrix (let's denote it  with
    \code{retval}) are ordered
    counterclockwise with the first vertex taken
    to be the one with smallest index.  Thus,
    \code{retval[i,"node2"]} and \code{retval[i,"node3"]} are larger
    than
    \code{retval[i,"node3"]} and index adjacent neighbors of
    node \code{retval[i,"node1"]}. The columns \code{trx} and
    \code{arcx}, x=1,2,3 index the triangle and arc,
    respectively, which are opposite (not shared
    by) node \code{nodex}, with \code{trix}= 0 if
    \code{arcx} indexes a boundary arc.  Vertex
    indexes range from 1 to N, triangle indexes
    from 0 to NT, and, if included, arc indexes
    from 1 to NA = NT+N-1.  The triangles are 
    ordered on first (smallest) vertex indexes,
    except that the sets of constraint triangles
    (triangles contained in the closure of a constraint
    region) follow the non-constraint
    triangles.
}
\value{A matrix with columns \code{node1},\code{node2},\code{node3},
  representing the vertex nodal indexes,
  \code{tr1},\code{tr2},\code{tr3}, representing neighboring triangle
  indexes and \code{arc1},\code{arc2},\code{arc3} reresenting arc indexes.

  Each row represents one triangle. 
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\seealso{\code{\link{tri}}, \code{\link{print.tri}}, \code{\link{plot.tri}}, \code{\link{summary.tri}}, \code{\link{triangles}}
}
\examples{
# use a slighlty modified version of data(tritest)
data(tritest2)
tritest2.tr<-tri.mesh(tritest2$x,tritest2$y)
triangles(tritest2.tr)
}
\keyword{spatial}

\eof
\name{sgeostat-internal}
\title{Internal functions}
\alias{voronoi.polyarea}
\alias{tri.vordist}
\alias{voronoi.findvertices}
\description{
 Internal tripack functions
}
\details{
  These functions are not intended to be called by the user.
 
}
\keyword{spatial}

\eof
\name{tritest}
\alias{tritest}
\alias{tritest2}
\non_function{}
\title{ tritest / sample data }
\description{
A very simply set set of points to test the tripack functions, taken
from the FORTRAN original. \code{tritest2} is a slight modification by 
adding \code{runif(,-0.1,0.1)} random numbers to the coordinates.
}
\references{
  R. J. Renka (1996). Algorithm 751: TRIPACK: a constrained
  two-dimensional {Delaunay} triangulation package.
  ACM Transactions on Mathematical Software.
  \bold{22}, 1-8.
}
\keyword{datasets}

\eof
\name{voronoi}
\title{Voronoi object}
\author{A. Gebhardt}
\alias{voronoi}
\arguments{
  \item{x,y}{x and y coordinates of nodes of the voronoi mosaic. Each node is a
    circumcircle center of some triangle from the Delaunay triangulation.}
  \item{node}{logical vector, indicating real nodes of the voronoi
  mosaic. These nodes are the centers of circumcircles of triangles with 
  positive area of the delaunay triangulation.

  If \code{node[i]}=\code{FALSE}, (\code{c[i]},\code{x[i]}) belongs to a 
  triangle with area 0.}
\item{n1,n2,n3}{indices of neighbour nodes. Negative indices indicate
  dummy points as neighbours.}
\item{tri}{triangulation object, see \code{\link{tri}}.}
\item{area}{area of triangle i. \code{area[i]}=-1 indicates a removed
  triangle with area 0 at the border of the triangulation.}
\item{ratio}{aspect ratio (inscribed radius/circumradius) of triangle
  i.}
\item{dummy.x,dummy.y}{x and y coordinates of dummy points. They are
  used for plotting of unbounded tiles.}
}
\description{
  An \code{voronoi} object is created with {\code{\link{voronoi.mosaic}}
}}
\seealso{\code{\link{voronoi.mosaic}},\code{\link{plot.voronoi}}
}
\keyword{spatial}


\eof
\name{voronoi.area}
\title{Calculate area of Voronoi polygons}
\author{S. J. Eglen}
\usage{voronoi.area(voronoi.obj)
}
\alias{voronoi.area}
\arguments{
  \item{voronoi.obj}{object of class \code{"voronoi"}}
}
\description{Computes the area of each Voronoi polygon.
For some sites at the edge of the region, the Voronoi polygon is not
bounded, and so the area of those sites cannot be calculated, and hence
will be NA.
}
\value{A vector of polygon areas.}
\seealso{
  \code{\link{voronoi}},
}
\keyword{spatial}

\examples{
data(tritest)
tritest.vm <- voronoi.mosaic(tritest$x,tritest$y)
tritest.vm.areas <- voronoi.area(tritest.vm)
plot(tritest.vm)
text(tritest$x, tritest$y, tritest.vm.areas)
}


\eof
\name{voronoi.findrejectsites}
\title{Find the Voronoi sites at the border of the region (to be rejected).}
\author{S. J. Eglen}
\usage{voronoi.findrejectsites(voronoi.obj, xmin, xmax, ymin, ymax)
}
\alias{voronoi.findrejectsites}
\arguments{
  \item{voronoi.obj}{object of class \code{"voronoi"}}
  \item{xmin}{minimum x-coordinate of sites in the region}
  \item{xmax}{maximum x-coordinate of sites in the region}
  \item{ymin}{minimum y-coordinate of sites in the region}
  \item{ymax}{maximum y-coordinate of sites in the region}

}
\description{Find the sites in the Voronoi tesselation that lie at the
  edge of the region.  A site is at the edge if any of the vertices of
  its Voronoi polygon lie outside the rectangle with corners (xmin,ymin)
  and (xmax,ymax).}
}
\value{A logical vector of the same length as the number of sites.  If
  the site is a reject, the corresponding element of the vector is set
  to TRUE.}
\seealso{
  \code{\link{tri.dellens}}
}
\keyword{spatial}


\eof
\name{voronoi.mosaic}
\title{Create a Voronoi mosaic}
\author{A. Gebhardt}
\usage{voronoi.mosaic(x,y=NULL,duplicate="error")
}
\alias{voronoi.mosaic}
\arguments{
  \item{x}{vector containing x coordinates of the data. If \code{y} is
    missing    \code{x} should contain two elements \code{$x} and \code{$y}.
  }
  \item{y}{vector containing y coordinates of the data.
  }
  \item{duplicate}{flag indicating how to handle duplicate elements.
   Possible values are: \code{"error"} -- default, \code{"strip"} --
   remove all duplicate points, \code{"remove"} -- leave one point of
   duplicate points. 
 }
}
\description{
  This function creates a Voronoi mosaic.

  It creates first a Delaunay triangulation, determines the circumcircle 
  centers of its triangles, and connects these points according to the
  neighbourhood relations between the triangles.
}
\value{
  An object of class \code{\link{voronoi}}.
}
\seealso{  \code{\link{voronoi}},\code{\link{voronoi.mosaic}},      \code{\link{print.voronoi}}, \code{\link{plot.voronoi}}
}
\examples{
# example from TRIPACK:
data(tritest)
tritest.vm<-voronoi.mosaic(tritest$x,tritest$y)
tritest.vm
# use a part of the quakes data set:
data(quakes)
quakes.part<-quakes[(quakes[,1]<=-17 & quakes[,1]>=-19.0 &
                     quakes[,2]<=182.0 & quakes[,2]>=180.0),]
quakes.vm<-voronoi.mosaic(quakes.part$lon, quakes.part$lat, duplicate="remove")
quakes.vm
}
\keyword{spatial}

\eof
\name{voronoi.polygons}
\alias{voronoi.polygons}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{ extract polygons from a voronoi mosaic }
\description{
  This functions extracts polygons from a \code{voronoi.mosaic} object.
}
\usage{
voronoi.polygons(voronoi.obj)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{voronoi.obj}{ object of class \code{voronoi.mosaic} }
}
\value{
  Returns an object of class \code{voronoi.polygons} with unamed list
  elements for each polygon. These list
  elements are matrices with columns \code{x} and \code{y}.
}
\author{ Denis White }
\seealso{ \code{\link{plot.voronoi.polygons}},\code{\link{voronoi.mosaic}}}

\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	     or do  help(data=index)  for the standard data sets.

data(tritest)
tritest.vm <- voronoi.mosaic(tritest$x,tritest$y)
tritest.vp <- voronoi.polygons(tritest.vm)
tritest.vp
}
\keyword{ spatial }

\eof
